"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.canAddToGroup = void 0;
const minimatch_1 = require("minimatch");
const is_non_empty_array_1 = require("tightrope/guard/is-non-empty-array");
function canAddToGroup(packageJsonFilesByName, group, instance) {
    const { dependencies, dependencyTypes, packages } = group.config;
    return (group.canAdd(instance) &&
        matchesDependencyTypes(dependencyTypes, instance) &&
        matchesPackages(packages, instance) &&
        matchesDependencies(packageJsonFilesByName, group, dependencies, instance));
}
exports.canAddToGroup = canAddToGroup;
function matchesDependencies(packageJsonFilesByName, group, dependencies, instance) {
    // matches if not defined
    if (!(0, is_non_empty_array_1.isNonEmptyArray)(dependencies))
        return true;
    return dependencies.some((pattern) => (pattern === '$LOCAL' &&
        instance.name in packageJsonFilesByName &&
        ((group.groupType === 'versionGroup' && instance.versionGroup === null) ||
            (group.groupType === 'semverGroup' && instance.semverGroup === null))) ||
        (0, minimatch_1.minimatch)(instance.name, pattern));
}
function matchesPackages(packages, instance) {
    // matches if not defined
    if (!(0, is_non_empty_array_1.isNonEmptyArray)(packages))
        return true;
    return packages.some((pattern) => (0, minimatch_1.minimatch)(instance.pkgName, pattern));
}
function matchesDependencyTypes(dependencyTypes, instance) {
    // matches if not defined
    if (!(0, is_non_empty_array_1.isNonEmptyArray)(dependencyTypes))
        return true;
    if (dependencyTypes.join('') === '**')
        return true;
    const negative = [];
    const positive = [];
    dependencyTypes.forEach((name) => {
        if (name.startsWith('!')) {
            negative.push(name.replace('!', ''));
        }
        else {
            positive.push(name);
        }
    });
    if ((0, is_non_empty_array_1.isNonEmptyArray)(negative) && !negative.includes(instance.strategy.name))
        return true;
    return (0, is_non_empty_array_1.isNonEmptyArray)(positive) && positive.includes(instance.strategy.name);
}
